r'''
Copy with a memory.

(c) Tom de Geus, 2021, MIT
'''

from ._version import *
from . import checksum
from . import convert
from . import detail
from . import path
from . import relpath
from . import rich
from . import rsync
from . import scp
from . import ssh
from . import yaml






